// This software is not affiliated with Valve Corporation
//   We are not affiliated, associated, authorized, endorsed by, or in any way officially 
//   connected with Valve Corporation, or any of its subsidiaries or its affiliates. 
// 
//   All trademarks are property of their respective owners

// Valve pak file directory reader

// VPK
//=======================================================================================================================

typedef struct VPKHeader VPKHeader;
typedef struct VPKDirectoryEntry VPKDirectoryEntry;

// API
// ---

// There is no load function, reading the header file as binary is enough.
VPKDirectoryEntry *vpk_find( VPKHeader *self, const char *asset );
void vpk_free( VPKHeader *self );

#pragma pack(push, 1)
struct VPKHeader
{
	u32 Signature;
	u32 Version;
	u32 TreeSize;
	u32 FileDataSectionSize;
	u32 ArchiveMD5SectionSize;
	u32 OtherMD5SectionSize;
	u32 SignatureSectionSize;
};

struct VPKDirectoryEntry
{
	u32 CRC;
	u16 PreloadBytes; 
	u16 ArchiveIndex;
	u32 EntryOffset;
	u32 EntryLength;
	u16 Terminator;
};
#pragma pack(pop)

#ifdef VALVE_IMPLEMENTATION

void vpk_free( VPKHeader *self )
{
	free( self );
}

VPKDirectoryEntry *vpk_find( VPKHeader *self, const char *asset )
{
	if( !self ) 
		return NULL;
	
	char wbuf[ 512 ];
	strcpy( wbuf, asset );
	
	char *ext = csr_findext( wbuf, '.' );
	*(ext-1) = 0x00;
	char *fn = csr_findext( wbuf, '/' );
	*(fn-1) = 0x00;
	char *dir = wbuf;
	
	char *pCur = ((char *)self) + sizeof( VPKHeader );
	
	while( 1 )
	{
		if( !*pCur ) break;

		int bExt = !strcmp( ext, pCur );		
		
		while( *( pCur ++ ) ) {};
		while( 1 )
		{
			if( !*pCur ) { pCur ++; break; }

			int bDir = !strcmp( dir, pCur );

			while( *( pCur ++ ) ) {};
			while( 1 )
			{
				if( !*pCur ) { pCur ++; break; }
			
				const char *vpk_fn = pCur;
			
				while( *( pCur ++ ) ) {};
				VPKDirectoryEntry *entry = (VPKDirectoryEntry *)pCur;
				
				if( !strcmp( vpk_fn, fn ) && bExt && bDir ) 
				{
					return entry;
				}
				
				pCur += entry->PreloadBytes + sizeof( VPKDirectoryEntry );
			}
			
			if( bDir && bExt ) return NULL;
		}
		
		if( bExt ) return NULL;
	}
	
	return NULL;
}

#endif
